 
/**************************************************************************
 *                                                                        *
 * This code is developed by Adam Li.  This software is an                *
 * implementation of a part of one or more MPEG-4 Video tools as          *
 * specified in ISO/IEC 14496-2 standard.  Those intending to use this    *
 * software module in hardware or software products are advised that its  *
 * use may infringe existing patents or copyrights, and any such use      *
 * would be at such party's own risk.  The original developer of this     *
 * software module and his/her company, and subsequent editors and their  *
 * companies (including Project Mayo), will have no liability for use of  *
 * this software or modifications or derivatives thereof.                 *
 *                                                                        *
 * Project Mayo gives users of the Codec a license to this software       *
 * module or modifications thereof for use in hardware or software        *
 * products claiming conformance to the MPEG-4 Video Standard as          *
 * described in the Open DivX license.                                    *
 *                                                                        *
 * The complete Open DivX license can be found at                         *
 * http://www.projectmayo.com/opendivx/license.php .                      *
 *                                                                        *
 **************************************************************************/

/************************************************************************
 *
 *  enc_engine.cpp, Encoder Engines
 *
 *  Copyright (C) 2000  DivX Networks
 * 
 *  Adam Li
 *  Cyril
 *
 *  DivX Advance Research Center <darc@projectmayo.com>
 *
 ************************************************************************/

// This file contains the encoder engine functions. There are starting the
// encoder engine, converting the image information to the correct format,
// calling function to compress frames, and ending the encoder engine.

#include "stdafx.h"
#include "codec.h"

#ifndef _ENCORE2_
#include "encore.h"

#else

#include "encore2.h"

#endif

static FILE *bf;

long codec::encBegin(LPARAM lParam1, LPARAM lParam2)
{
	BITMAPINFO *lpbiInput;
	BITMAPV4HEADER *infohdr_i;
	ENC_PARAM param1;

	lpbiInput = (BITMAPINFO *)lParam1;
	infohdr_i = (BITMAPV4HEADER *)&(lpbiInput->bmiHeader);


	memset(&param1, 0, sizeof(param1));
	param1.x_dim = infohdr_i->bV4Width;
	param1.y_dim = infohdr_i->bV4Height;
	param1.bitrate = m_stSTATE_DATA.bitrate;
	param1.rc_period = m_stSTATE_DATA.rc_period;
	param1.rc_reaction_period = m_stSTATE_DATA.rc_reaction_period;
	param1.rc_reaction_ratio = m_stSTATE_DATA.rc_reaction_ratio;
	param1.framerate = framerate;
	param1.max_key_interval = m_stSTATE_DATA.max_key_interval;
	param1.max_quantizer = m_stSTATE_DATA.max_quantizer;
	param1.min_quantizer = m_stSTATE_DATA.min_quantizer;

#ifndef _ENCORE2_
	param1.search_range = m_stSTATE_DATA.search_range;

#else

	param1.quality = m_stSTATE_DATA.quality;

#endif

/***
	param1.bitrate = m_stSTATE_DATA.bitrate;
	param1.rc_period = m_stSTATE_DATA.rc_period;
	param1.rc_reaction_period = m_stSTATE_DATA.rc_reaction_period;
	param1.rc_reaction_ratio = m_stSTATE_DATA.rc_reaction_ratio;
	param1.framerate = m_stSTATE_DATA.framerate;
	param1.max_key_interval = m_stSTATE_DATA.max_key_interval;
	param1.max_quantizer = m_stSTATE_DATA.max_quantizer;
	param1.min_quantizer = m_stSTATE_DATA.min_quantizer;
	param1.search_range = m_stSTATE_DATA.search_range;
***/
 
	// initialize the encore()
	encore(m_hEnc, ENC_OPT_INIT, &param1, NULL);
#ifdef _ENCORE2_

	m_hEnc=param1.handle;

	if(!m_hEnc)

		return ICERR_BADFORMAT;

#endif
	#ifdef _WRITE_
	bf = fopen("develop_1_0_5.bis", "wb");
	#endif

	return ICERR_OK;
}

long codec::encEnd(LPARAM lParam1, LPARAM lParam2)
{
	encore(m_hEnc, ENC_OPT_RELEASE, NULL, NULL);

	#ifdef _WRITE_

	if(bf)
		fclose(bf);
	#endif


	if(!bf)

		return ICERR_ERROR;
	return ICERR_OK;
}

long codec::encEncode(LPARAM lParam1, LPARAM lParam2)
{
	ICCOMPRESS *icc = (ICCOMPRESS *)lParam1;
	int iccSize = (int)lParam2;
	BITMAPV4HEADER *lpbihdr_i, *lpbihdr_o;
	ENC_FRAME frame;
	ENC_RESULT outflag;
 
	long imgtype, x_dim, y_dim, size, flagmem = 0;

	long enc_opt = 0;

	long result;

	lpbihdr_i = (BITMAPV4HEADER *)icc->lpbiInput;
	lpbihdr_o = (BITMAPV4HEADER *)icc->lpbiOutput;

	imgtype = getImageType(lpbihdr_i);
	if (imgtype == 0) return ICERR_ERROR;

	// call encore()
	x_dim = lpbihdr_i->bV4Width;
	y_dim = lpbihdr_o->bV4Height;
	size = x_dim * y_dim * 3 / 2;

#ifndef _ENCORE2_
	if ((imgtype == 64) || (imgtype == 65)) {	// planar image
		frame.image = icc->lpInput;
		convertImage(icc->lpInput, frame.image, imgtype, x_dim, y_dim);
	} else {									// packed image
		if ((frame.image = malloc(size)) == NULL) return ICERR_ERROR;
		flagmem = 1;
		convertImage(icc->lpInput, frame.image, imgtype, x_dim, y_dim);
	}

#else

	switch(imgtype)

	{

	case 65: // IYUV. not 64?

		frame.image=icc->lpInput;

		frame.colorspace=ENC_CSP_YV12;

		break;

	case 17: // RGB24

		frame.image=icc->lpInput;

		frame.colorspace=ENC_CSP_RGB24;

		break;

	case 64: // YV12

		frame.image = icc->lpInput;

		convertImage(frame.image, frame.image, imgtype, x_dim, y_dim);

		frame.colorspace=ENC_CSP_YV12;

		break;

	default:

		if ((frame.image = malloc(size)) == NULL) return ICERR_ERROR;

		flagmem = 1;

		convertImage(icc->lpInput, frame.image, imgtype, x_dim, y_dim);

		frame.colorspace=ENC_CSP_YV12;

		break;

	}
#endif
	frame.bitstream = icc->lpOutput;
	frame.length = 0;

	// encode the frame with encore

	result = encore(m_hEnc, 
#ifndef _ENCORE2_

		enc_opt, 

#else

		ENC_OPT_ENCODE,

#endif
		&frame,
		&outflag);

	// set the length of the output bitstream
	lpbihdr_o->bV4SizeImage = frame.length;
	// set the flag so Windows knows it is a key frame
	*(icc->lpdwFlags) = 0;

#ifndef _ENCORE2_
	if (outflag.isKeyFrame) 

#else

	if (outflag.is_key_frame)

#endif
		*(icc->lpdwFlags) = AVIIF_KEYFRAME;

	#ifdef _WRITE_
	fwrite(frame.bitstream, 1, frame.length, bf);
	#endif

	if (flagmem) free(frame.image);

	return ICERR_OK;
}

