
/**************************************************************************
 *                                                                        *
 * This code is developed by Adam Li.  This software is an                *
 * implementation of a part of one or more MPEG-4 Video tools as          *
 * specified in ISO/IEC 14496-2 standard.  Those intending to use this    *
 * software module in hardware or software products are advised that its  *
 * use may infringe existing patents or copyrights, and any such use      *
 * would be at such party's own risk.  The original developer of this     *
 * software module and his/her company, and subsequent editors and their  *
 * companies (including Project Mayo), will have no liability for use of  *
 * this software or modifications or derivatives thereof.                 *
 *                                                                        *
 * Project Mayo gives users of the Codec a license to this software       *
 * module or modifications thereof for use in hardware or software        *
 * products claiming conformance to the MPEG-4 Video Standard as          *
 * described in the Open DivX license.                                    *
 *                                                                        *
 * The complete Open DivX license can be found at                         *
 * http://www.projectmayo.com/opendivx/license.php .                      *
 *                                                                        *
 **************************************************************************/

/*************************************************************************
 *
 *  Maintenance.cpp, Maintenance functions
 *
 *  Copyright (C) 2000  DivX Networks
 * 
 *  Adam Li
 *  Andrea Graziani
 *  Cyril
 *
 *  DivX Advance Research Center <darc@projectmayo.com>
 *
 ************************************************************************/

// This includes the maintance functions of the driver. There are functions
// that answers the various info query messages, and configuration of 
// the codec.

#include "stdafx.h"
#include "codec.h"
#include "resource.h"
#include "config_dialog.h"

#include "decore.h"

#ifndef _ENCORE2_
#include "encore.h"

#else

#include "encore2.h"

#endif

codec::codec()
{
	// instantiate using global defaults
/*

	m_stSTATE_DATA.bitrate = bitrate;
	m_stSTATE_DATA.rc_period = rc_period;
	m_stSTATE_DATA.max_quantizer = max_quantizer;
	m_stSTATE_DATA.min_quantizer = min_quantizer;
	m_stSTATE_DATA.search_range = search_range;
*/
	m_bDecompressBeginDone = false;

#ifndef _ENCORE2_

	m_hEnc=(unsigned long)this;

#else

	m_hEnc=0;

#endif
}

codec::~codec()
{
  if ( m_bDecompressBeginDone ) {
    decore((long) this, DEC_OPT_RELEASE, NULL, NULL);
      m_bDecompressBeginDone = false;
  }
#ifndef _ENCORE2_
	encore((long)this, ENC_OPT_RELEASE, NULL, NULL);

#endif
}

long codec::getInfo(LPARAM lParam1, LPARAM lParam2)
{
	ICINFO *info = (ICINFO *)lParam1;

	info->fccType = ICTYPE_VIDEO;
	info->fccHandler = mmioFOURCC('D', 'I', 'V', 'X');
	info->dwFlags = 
		VIDCF_FASTTEMPORALC | VIDCF_FASTTEMPORALD | VIDCF_COMPRESSFRAMES; 
	info->dwVersion = 0;
	info->dwVersionICM = ICVERSION;
	wcscpy(info->szName, L"DivX codec"); 

#ifndef _ENCORE2_
	wcscpy(info->szDescription, L"DivX MPEG-4 Codec (D 1.0.5n)");

#else

	wcscpy(info->szDescription, L"DivX MPEG-4 Codec (2.0 Beta 1)");

#endif

	return lParam2;
}

long codec::about(LPARAM lParam1, LPARAM lParam2)
{
	if (lParam1 == -1) return ICERR_OK;
	CDialog dlgAbout(IDD_DIALOG_ABOUT, NULL/*&wndParent*/);
	dlgAbout.DoModal();
	return ICERR_OK;
}

long codec::config(LPARAM lParam1, LPARAM lParam2)
{
	if (lParam1 == -1) return ICERR_OK;

	ConfigDialog dlgConfig(NULL/*&wndParent*/);
	dlgConfig.m_bitrate = m_stSTATE_DATA.bitrate / 1000;
	dlgConfig.m_rc_period = m_stSTATE_DATA.rc_period;
	dlgConfig.m_rc_reaction_period = m_stSTATE_DATA.rc_reaction_period;
	dlgConfig.m_rc_reaction_ratio = m_stSTATE_DATA.rc_reaction_ratio;
	dlgConfig.m_max_quant = m_stSTATE_DATA.max_quantizer;
	dlgConfig.m_min_quant = m_stSTATE_DATA.min_quantizer;
	dlgConfig.m_key_interval = m_stSTATE_DATA.max_key_interval;
	dlgConfig.m_search_range.Format("%d", m_stSTATE_DATA.search_range);

	dlgConfig.m_iQuality = m_stSTATE_DATA.quality-1;
	if (dlgConfig.DoModal() == IDOK) {
		m_stSTATE_DATA.bitrate = dlgConfig.m_bitrate * 1000;
		m_stSTATE_DATA.rc_period = dlgConfig.m_rc_period;
		m_stSTATE_DATA.rc_reaction_period = dlgConfig.m_rc_reaction_period;
		m_stSTATE_DATA.rc_reaction_ratio = dlgConfig.m_rc_reaction_ratio;
		m_stSTATE_DATA.max_quantizer = dlgConfig.m_max_quant;
		m_stSTATE_DATA.min_quantizer = dlgConfig.m_min_quant;
		m_stSTATE_DATA.max_key_interval = dlgConfig.m_key_interval;
		m_stSTATE_DATA.search_range = atoi(dlgConfig.m_search_range);

		m_stSTATE_DATA.quality = dlgConfig.m_iQuality+1;
	};

/***
	ConfigDialog dlgConfig(NULL);
	dlgConfig.m_bitrate = m_stSTATE_DATA.bitrate / 1000;
	dlgConfig.m_rc_period = m_stSTATE_DATA.rc_period;
	dlgConfig.m_rc_reaction_period = m_stSTATE_DATA.rc_reaction_period;
	dlgConfig.m_rc_reaction_ratio = m_stSTATE_DATA.rc_reaction_ratio;
	dlgConfig.m_max_quant = m_stSTATE_DATA.max_quantizer;
	dlgConfig.m_min_quant = m_stSTATE_DATA.min_quantizer;
	dlgConfig.m_key_interval = m_stSTATE_DATA.max_key_interval;
	dlgConfig.m_search_range.Format("%d", m_stSTATE_DATA.search_range);
	if (dlgConfig.DoModal() == IDOK) {
		m_stSTATE_DATA.bitrate = dlgConfig.m_bitrate * 1000;
		m_stSTATE_DATA.rc_period = dlgConfig.m_rc_period;
		m_stSTATE_DATA.rc_reaction_period = dlgConfig.m_rc_reaction_period;
		m_stSTATE_DATA.rc_reaction_ratio = dlgConfig.m_rc_reaction_ratio;
		m_stSTATE_DATA.max_quantizer = dlgConfig.m_max_quant;
		m_stSTATE_DATA.min_quantizer = dlgConfig.m_min_quant;
		m_stSTATE_DATA.max_key_interval = dlgConfig.m_key_interval;
		m_stSTATE_DATA.search_range = atoi(dlgConfig.m_search_range);
	}
***/

	return ICERR_OK;
}

long codec::encGetFormat(LPARAM lParam1, LPARAM lParam2)
{
	BITMAPINFO *lpbiInput, *lpbiOutput;
	BITMAPINFOHEADER *infohdr;
	long x_dim, y_dim;

	lpbiInput = (BITMAPINFO *)lParam1;
	lpbiOutput = (BITMAPINFO *)lParam2;


	FILE* f=fopen("c:\msg.log", "ab");

	fprintf(f, "ICM_COMPRESS_GET_FORMAT\n");

	fprintf(f, "Input:\n x_dim %d, y_dim %d, compression %d, bits %d\n",

			lpbiInput->bmiHeader.biWidth,

			lpbiInput->bmiHeader.biHeight,

			lpbiInput->bmiHeader.biCompression,

			lpbiInput->bmiHeader.biBitCount

			);

	fclose(f);


//	if (lpbiOutput == NULL) return(sizeof(BITMAPV4HEADER) + sizeof(RGBQUAD));
	if (lpbiOutput == NULL) return sizeof(BITMAPINFO);



	// everything is copied from the input, including the dimensions
	*lpbiOutput = *lpbiInput;
	infohdr = (BITMAPINFOHEADER *)&(lpbiOutput->bmiHeader);
	x_dim = infohdr->biWidth;
	y_dim = infohdr->biHeight;

//	infohdr->biBitCount = 0;
	infohdr->biCompression = FOURCC_DIVX;

//	infohdr->biSize = sizeof(BITMAPINFOHEADER);
	infohdr->biSizeImage = x_dim * y_dim * 3 / 2 * sizeof(short int);
	return(ICERR_OK);
}

long codec::encGetSize(LPARAM lParam1, LPARAM lParam2)
{
	BITMAPINFO *lpbiInput, *lpbiOutput;
	BITMAPV4HEADER *infohdr;
	long x_dim, y_dim;

	lpbiInput = (BITMAPINFO *)lParam1;
	lpbiOutput = (BITMAPINFO *)lParam2;

	infohdr = (BITMAPV4HEADER *)&(lpbiOutput->bmiHeader);
	x_dim = infohdr->bV4Width;
	y_dim = infohdr->bV4Height;

	return(x_dim * y_dim * 3 / 2 * sizeof(short int));
}

long codec::encQuery(LPARAM lParam1, LPARAM lParam2)
{
	BITMAPINFO *lpbiInput, *lpbiOutput;
	BITMAPV4HEADER *infohdr_i, *infohdr_o;
	long x_dim_i, y_dim_i, x_dim_o, y_dim_o;

	lpbiInput = (BITMAPINFO *)lParam1;
	lpbiOutput = (BITMAPINFO *)lParam2;



	infohdr_i = (BITMAPV4HEADER *)&(lpbiInput->bmiHeader);
	x_dim_i = infohdr_i->bV4Width;
	y_dim_i = infohdr_i->bV4Height;
	FILE* f=fopen("c:\msg.log", "ab");

	fprintf(f, "ICM_COMPRESS_QUERY\n");

	fprintf(f, "Input:\n x_dim %d, y_dim %d, compression %d, bits %d\n",

			x_dim_i, y_dim_i, infohdr_i->bV4V4Compression,

			infohdr_i->bV4BitCount);

	fclose(f);


	// input dimension has to be divicable by 16
	if ((x_dim_i % 16) || (y_dim_i % 16)) return (ICERR_BADFORMAT);
	// check to see if the input format is supported
	if (getImageType(infohdr_i) == 0) return (ICERR_BADFORMAT);

	if (lpbiOutput == NULL) return (ICERR_OK);

	infohdr_o = (BITMAPV4HEADER *)&(lpbiOutput->bmiHeader);
	x_dim_o = infohdr_o->bV4Width;
	y_dim_o = infohdr_o->bV4Height;


	f=fopen("c:\msg.log", "ab");

	fprintf(f, "Output:\n x_dim %d, y_dim %d, compression %d, bits %d\n",

			x_dim_o, y_dim_o, infohdr_o->bV4V4Compression,

			infohdr_o->bV4BitCount);

	fclose(f);


	// output dimension must be exactly same as the input dimension
	if ((x_dim_o != x_dim_i) || (y_dim_o != y_dim_i)) return (ICERR_BADFORMAT);
	// output format must be DivX with BitCount 0
	if ((infohdr_o->bV4V4Compression != FOURCC_DIVX)
		|| (infohdr_o->bV4BitCount != 0)) return (ICERR_BADFORMAT);

	return(ICERR_OK);
}

long codec::encFramesInfo(LPARAM lParam1, LPARAM lParam2)
{
	ICCOMPRESSFRAMES *icf;
	long icfSize;

	icf = (ICCOMPRESSFRAMES *)lParam1;
	icfSize = (long)lParam2;

	framerate = (float)icf->dwRate / (float)icf->dwScale;

	return(ICERR_OK);
}

long codec::decGetFormat(LPARAM lParam1, LPARAM lParam2)
{
	BITMAPINFO *lpbiInput, *lpbiOutput;
	BITMAPV4HEADER *infohdr;
	long x_dim, y_dim;

	lpbiInput = (BITMAPINFO *)lParam1;
	lpbiOutput = (BITMAPINFO *)lParam2;

	if (lpbiOutput == 0)
		return (sizeof(BITMAPV4HEADER) + sizeof(RGBQUAD));

	// everything is copied from the input, including the dimensions
	*lpbiOutput = *lpbiInput;
	infohdr = (BITMAPV4HEADER *)&(lpbiOutput->bmiHeader);
	x_dim = infohdr->bV4Width;
	y_dim = infohdr->bV4Height;

	infohdr->bV4BitCount = 24;
	infohdr->bV4V4Compression = BI_RGB;
	infohdr->bV4SizeImage = x_dim * y_dim * 3;

	return ICERR_OK;
}

long codec::decQuery(LPARAM lParam1, LPARAM lParam2)
{
	BITMAPINFO *lpbiInput, *lpbiOutput;
	BITMAPV4HEADER *infohdr_i, *infohdr_o;
	long x_dim_i, y_dim_i, x_dim_o, y_dim_o;

	char * pnColor;
	unsigned long * pdwColor;
	FOURCC_LOOKUP * pFOURCC_LOOKUP;

	lpbiInput = (BITMAPINFO *)lParam1;
	lpbiOutput = (BITMAPINFO *)lParam2;

	infohdr_i = (BITMAPV4HEADER *)&(lpbiInput->bmiHeader);
	x_dim_i = infohdr_i->bV4Width;
	y_dim_i = infohdr_i->bV4Height;

	// input supported features
	if ((x_dim_i % 16) || (y_dim_i % 16)) 
		return ICERR_BADFORMAT; // input dimension has to be divicable by 16
	if (!((infohdr_i->bV4V4Compression == FOURCC_DIVX) || (infohdr_i->bV4V4Compression == 4L /* BI_JPEG */))
		|| (infohdr_i->bV4BitCount != 0)) 
		return ICERR_BADFORMAT; // input format must be DivX with BitCount 0

	if (lpbiOutput == NULL) 
		return ICERR_OK;

	infohdr_o = (BITMAPV4HEADER *)&(lpbiOutput->bmiHeader);
	x_dim_o = infohdr_o->bV4Width;
	y_dim_o = infohdr_o->bV4Height;

	// output supported features - cwk ignore sign of biHeight
	if ((x_dim_o != x_dim_i) || ( abs(y_dim_o) != y_dim_i)) 
		return ICERR_BADFORMAT; // output dimension must be exactly same as the input dimension

	if (infohdr_o->bV4V4Compression == BI_RGB) {
    if ( infohdr_o->bV4BitCount == 32 ) {
      // rgb32 + or - biHeight
      return ICERR_OK;
    }
    if ( infohdr_o->bV4BitCount == 24 ) {
      // rgb24 + or - biHeight
      return ICERR_OK;
    }
    if ( infohdr_o->bV4BitCount == 16 ) {
      // rgb555 + or - biHeight
      return ICERR_OK;
    }
    return ICERR_BADFORMAT;
  } 
	else if (infohdr_o->bV4V4Compression == BI_BITFIELDS) 
	{
    if ( infohdr_o->bV4BitCount != 16 ) {
      return ICERR_BADFORMAT;
    }
    // this is the way MS docs says pointer to RGBQUAD should be computed
		pnColor = ((LPSTR)lpbiOutput + (WORD)(lpbiOutput->bmiHeader.biSize)); 
    pdwColor = (unsigned long*)pnColor;
    if ( pdwColor[0] != 0xf800 ) {
    return ICERR_BADFORMAT;
    }
    if ( pdwColor[1] != 0x07e0 ) {
    return ICERR_BADFORMAT;
    }
    if ( pdwColor[2] != 0x001f ) {
    return ICERR_BADFORMAT;
    }
    // rgb565 + or - biHeight
    return ICERR_OK;
  } 
	else 
	{
    pFOURCC_LOOKUP = dec_lookup_table;
    while ( pFOURCC_LOOKUP->nFourCC ) {
      if ( infohdr_o->bV4V4Compression == pFOURCC_LOOKUP->nFourCC ) {
          return ICERR_OK;
      }
      pFOURCC_LOOKUP++;
    }
		return ICERR_BADFORMAT;
  }

	return ICERR_OK;
}

